<?php

namespace Vialibri\HarvestPlugin;

use Vialibri\HarvestFormatting\FormatException;
use Vialibri\HarvestFormatting\XmlOutputter;

class RequestHandler {

  private const DEFAULT_PAGE_SIZE = 100;

  public static function ids() {

    self::handleXmlRequest(function (DataFetcher $fetcher, XmlOutputter $outputter) {
      $outputter->outputIdXml(
        $fetcher->ids(),
        $fetcher->lastUpdateDate()
      );
    });

  }

  public static function bookData() {
    self::handleXmlRequest(function (DataFetcher $fetcher, XmlOutputter $outputter) {
      $outputter->outputBookXml(
        $fetcher->books(self::page(), self::pageSize()),
        self::dataUrlForPage(self::page() + 1, self::pageSize())
      );
    });

  }

  public static function idsAsJson() {
    self::handleJsonRequest(function (DataFetcher $fetcher) {
      return $fetcher->ids();
    });
  }

  public static function bookDataAsJson() {
    self::handleJsonRequest(function (DataFetcher $fetcher) {
      return $fetcher->books(self::page(), self::pageSize());
    });
  }

  private static function page(): int
  {
    return !empty($_GET['harvest-page']) && is_numeric($_GET['harvest-page'])
              ? intval($_GET['harvest-page'])
              : 1;
  }

  private static function pageSize(): int
  {
    return !empty($_GET['harvest-page-size']) && is_numeric($_GET['harvest-page-size'])
      ? intval($_GET['harvest-page-size'])
      : self::DEFAULT_PAGE_SIZE;
  }

  private static function dataUrlForPage($pageNumber, $pageSize) {
    return add_query_arg(
      array(
        'action' => 'vialibri_harvest_book_data',
        'harvest-page-size' => $pageSize === self::DEFAULT_PAGE_SIZE ? null : $pageSize,
        'harvest-page' => $pageNumber
      ),
      admin_url('admin-ajax.php')
    );
  }

  private static function handleJsonRequest($getDataFn) {

    // Allow longer to process the script. This won't work on some hosts though.
    set_time_limit(300);

    $fetcher = new DataFetcher();

    header('Content-Type: application/json');

    if(current_user_can('manage_options')) {
      try {
        echo json_encode($getDataFn($fetcher));
      } catch (FormatException $ex) {
        echo json_encode($ex->getMessage());
      } catch (\Exception $ex) {
        echo json_encode(
          'The following error was encountered when fetching data using a filter: '
            .$ex->getMessage()
        );
      }
    }

    die();
  }

  private static function handleXmlRequest($dataOutputFn) {

    // Allow longer to process the script. This won't work on some hosts though.
    set_time_limit(300);

    global $vharvest_plugin_version;

    $fetcher = new DataFetcher();
    $outputter = new XmlOutputter($fetcher->sources($vharvest_plugin_version));

    header('Content-Type: application/xml');

    try {
      $dataOutputFn($fetcher, $outputter);
    } catch (FormatException $ex) {
      $outputter->outputErrorXml($ex->getMessage());
    } catch (\Exception $ex) {
      $outputter->outputErrorXml(
        'The following error was encountered when fetching data using a filter: '
          .$ex->getMessage()
      );
    }

    die();
  }

}
