<?php

namespace Vialibri\HarvestPlugin;

class SettingsPage {

  public static function init() {
    add_options_page(
      'viaLibri Harvest Settings',
      'viaLibri Harvest',
      'manage_options',
      'vialibri-harvest-settings',
      array('Vialibri\HarvestPlugin\SettingsPage', 'renderPage')
    );
  }

  public static function enqueueScript($hook) {
    global $vharvest_plugin_version;

    if($hook === 'settings_page_vialibri-harvest-settings' && self::currentTabSlug() === 'books') {

      wp_enqueue_script(
        'vialibri-harvest-settings-script',
        plugins_url('/../assets/js/admin.js', __FILE__),
        array('jquery'),
        $vharvest_plugin_version
      );

      wp_enqueue_style(
        'vialibri-harvest-settings-style',
        plugins_url('/../assets/css/admin.css', __FILE__),
        array(),
        $vharvest_plugin_version
      );
    }
  }

  private static function tabs() {
    return apply_filters('vialibri_harvest_settings_tabs', array( 'books' => 'Books' ));
  }

  public static function currentTabSlug() {
    $tabs = self::tabs();
    $current_tab_slug = array_keys($tabs)[0];
    if(isset($_GET['tab']) && isset($tabs[$_GET['tab']])) {
      $current_tab_slug = $_GET['tab'];
    }

    return $current_tab_slug;
  }

  public static function renderPage() {
    $tabs = self::tabs();
    $current_tab_slug = self::currentTabSlug();
    ?>

    <div class="wrap">
      <h1>viaLibri Harvest</h1>

      <?php if(count($tabs) > 1) { ?>
        <h2 class="nav-tab-wrapper">
          <?php foreach( $tabs as $slug => $name ) {
            $class = $slug === $current_tab_slug ? ' nav-tab-active' : '';
            echo "<a class='nav-tab$class' href='?page=vialibri-harvest-settings&tab=$slug'>{$name}</a>";
          } ?>
        </h2>
      <?php } ?>


      <?php do_action('vialibri_harvest_settings_tab_'.$current_tab_slug); ?>

    </div>

    <?php
  }

  public static function renderBooksTab() {
    $bookUrl = add_query_arg(
      'action',
      'vialibri_harvest_book_data_as_json',
      admin_url('admin-ajax.php')
    );
    $idUrl = add_query_arg(
      'action',
      'vialibri_harvest_id_list_as_json',
      admin_url('admin-ajax.php')
    );
    ?>

    <ul id="filter-picker" class="subsubsub">
      <li><a href="#" data-filter="errors" class="current">Books with errors</a></li>
      <li><a href="#" data-filter="all">All books</a></li>
      <li><a href="#" data-filter="ids">IDs</a></li>
    </ul>
    <div id="harvest-data" data-url="<?php esc_attr_e($bookUrl); ?>">
      <p>
        <div class="spinner is-active"></div>
        Loading (<span id="harvest-data-count">0</span>/<span id="harvest-data-total">0</span>)...
      </p>
    </div>
    <div id="harvest-ids" data-url="<?php esc_attr_e($idUrl); ?>" style="display: none;">
      <p>
        These are the IDs that are passed to the <code>vialibri_harvest_id_list</code>:
      </p>
      <ul></ul>
    </div>
    <p id="no-books" style="display: none;">
      No books match this filter.
    </p>

    <?php
  }

}
