jQuery(function ($) {

  var $data = $('#harvest-data');
  var $ids = $('#harvest-ids');
  var $picker = $('#filter-picker');

  function renderBooks(books) {

    $picker.find('li a[data-filter="errors"]').append(
      ' ('+ $.grep(books, function (b) { return !!b.errors; }).length + ')'
    );
    $picker.find('li a[data-filter="all"]').append(
      ' ('+ books.length + ')'
    );

    $.each(books, function (i, book) {

      var $div = $('<div class="vlh-book"></div>');

      if(book.errors) {
        var $el = $('<div class="vlh-errors"><p>The following errors prevent this book from being listed by viaLibri:</p><ul></ul></div>');

        $el.children('ul').append(
          $.map(book.errors, function (e) {
            return $('<li></li>').text(e);
          })
        );

        $div.append($el);
      }

      $.each(book, function(key, value) {
        if(key !== 'errors') {
          var $el = $('<div class="vlh-data-row"></div>');
          value = formatValue(value);
          $el.append($('<div class="vlh-data-row__key"></div>').text(key));
          $el.append($('<div class="vlh-data-row__value"></div>').text(value));
          $div.append($el);
        }
      });

      $data.append($div);
    });
  }

  function loadBooks(url, pageNumber, allBooks, updateCb, finishedCb, failCb) {

    $.get(url + '&harvest-page=' + pageNumber, function (books) {

      if(typeof books === 'string') {
        finishedCb(books);
      } else if(books.length > 0) {
        allBooks = allBooks.concat(books);
        updateCb(allBooks.length);
        loadBooks(url, pageNumber + 1, allBooks, updateCb, finishedCb, failCb);
      } else {
        finishedCb(allBooks);
      }

    }).fail(failCb);
  }

  function renderIds(ids) {

    $picker.find('li a[data-filter="ids"]').append(
      ' ('+ ids.length + ')'
    );

    $('#harvest-data-total').text(ids.length);

    $ids.children('ul').append(
      $.map(ids, function (id) {
        return $('<li></li>').text(id);
      })
    );
  }

  function setFilter(filter) {
    $picker.find('a').removeClass('current');
    $picker.find('a[data-filter="'+ filter +'"]').addClass('current');

    if(filter === 'ids') {
      $ids.show();
      $data.hide();
      $('#no-books').hide();
    } else {
      if(filter === 'all') {
        $data.children('.vlh-book').show();
      } else {
        $data.children('.vlh-book').hide();
        $data.children('.vlh-book:has(.vlh-errors)').show();
      }

      $ids.hide();
      $data.show();
      $('#no-books').toggle($data.children(':visible').length === 0);
    }
  }

  function formatValue(value) {
    if(typeof value === 'null') {
      return '(null)';
    } else if(typeof value === 'object') {
      return JSON.stringify(value); // Mainly just arrays passed to keywords.
    } else if(value === '') {
      return '(empty string)';
    } else {
      return value;
    }
  }

  function displayError(text) {
    var $err = $('<div class="notice notice-error"><p></p></div>');
    $err.find('p').text(text);
    $data.html($err);
    $picker.hide();
  }

  function handleRequestError(xhr, status, error) {
    displayError(
      'An error was encountered when fetching the book data ("'+error+'"). '
      +'This could be due to a timeout or badly formatted code in the filters.'
    );
  }

  loadBooks(
    $data.data('url'),
    1,
    [],
    function (count) {
      $('#harvest-data-count').text(count);
    },
    function (books) {
      $data.empty();

      if(typeof books === 'string') {
        displayError(books);
      } else {
        renderBooks(books);
        setFilter('errors');
      }
    },
    handleRequestError
  );

  $.get($ids.data('url'), function (ids) {

    // We don't do anything for errors. These are taken care of by the other GET.
    if(typeof ids !== 'string') {
      renderIds(ids);
    }

  }).fail(handleRequestError);

  $picker.find('a').click(function () {
    setFilter($(this).data('filter'));
  });

});
