<?php

namespace Vialibri\HarvestFormatting;

abstract class XmlFormatter {

  private $sources;

  public function __construct(array $sources)
  {
    $this->sources = $sources;
  }

  protected abstract function configureDocumentOutput(\XMLWriter $writer);

  protected function formatIdXml(array $ids, $lastUpdateDate)
  {
    $writer = $this->setUpDoc();

    $writer->startElement('Sync_Data');

      $this->addSources($writer);

      $writer->startElement('update');
        $writer->startElement('date_update');
          $writer->text($this->formatValue($lastUpdateDate, 'date'));
        $writer->endElement();
      $writer->endElement();
      $writer->startElement('ID_set');
        foreach ($ids as $id) {
          $writer->startElement('id');
            $writer->text($id);
          $writer->endElement();
        }
      $writer->endElement();
    $writer->endElement();
    $writer->endDocument();

    return $writer;
  }

  public function formatBookXml(array $books, $nextPageUrl)
  {
    $writer = $this->setUpDoc();

    $writer->startElement('Books');
    $writer->writeAttribute('next-page', $nextPageUrl);

      $this->addSources($writer);

      foreach ($books as $book) {
        if(empty($book['errors'])) {
          $this->addIndividualBook($writer, $book);
        }
      }

    $writer->endElement();
    $writer->endDocument();

    return $writer;
  }

  public function formatErrorXml($text)
  {
    $writer = $this->setUpDoc();

    $writer->startElement('error');
      $writer->text($text);
    $writer->endElement();

    $writer->endDocument();

    return $writer;
  }

  private function addIndividualBook(\XMLWriter $writer, array $book)
  {
    $writer->startElement('Book');
    foreach ($book as $key => $value) {
      if($value !== '' && !is_null($value) && in_array($key, array_keys(FieldDetails::$fields))) {
        $dataType = FieldDetails::$fields[$key]['type'];

        $writer->startElement($key);
          $writer->text($this->formatValue($value, $dataType));
        $writer->endElement();
      }
    }

    $writer->endElement();
  }

  private function addSources(\XMLWriter $writer)
  {
    foreach ($this->sources as $source) {
      $writer->startElement('source');
        $writer->writeAttribute('software', $source['name']);
        $writer->writeAttribute('version', $source['version']);
      $writer->endElement();
    }
  }

  private function formatValue($val, $dataType)
  {
    if($dataType === 'boolean' && is_bool($val)) {
      return $val ? 'yes' : 'no';
    } elseif($dataType === 'date' && $val instanceof \DateTime) {
      return $val->format('Y-m-d H:i:s');
    } elseif($dataType === 'delimited' && is_array($val)) {
      return implode(', ', $val);
    } else {
      return $val;
    }
  }

  private function setUpDoc()
  {
    $writer = new \XMLWriter();
    $this->configureDocumentOutput($writer);
    $writer->setIndent(true);
    $writer->setIndentString('  ');
    $writer->startDocument('1.0', 'UTF-8');

    return $writer;
  }

}
