<?php

namespace Vialibri\HarvestFormatting\Tests\Unit;

use Vialibri\HarvestFormatting\DataValidator;

class DataValidatorTest extends \PHPUnit_Framework_TestCase
{

  public function testValidatedIds()
  {
    $subject = new DataValidator();

    $result = $subject->validatedIds(['ABC123', 1234]);

    $this->assertSame(['ABC123', 1234], $result);
  }

  /**
   * @expectedException \Vialibri\HarvestFormatting\FormatException
   * @expectedExceptionMessage The value passed as the ID list is not an array.
   */
  public function testValidatedIdsWithNonArray()
  {
    $subject = new DataValidator();
    $subject->validatedIds('Hello');
  }

  /**
   * @expectedException \Vialibri\HarvestFormatting\FormatException
   * @expectedExceptionMessage The value at index 2 in the ID array is an empty string.
   */
  public function testValidatedIdsWithEmptyString()
  {
    $subject = new DataValidator();
    $subject->validatedIds(['ggergre', 'wddwef', '']);
  }

  /**
   * @expectedException \Vialibri\HarvestFormatting\FormatException
   * @expectedExceptionMessage The value at index 0 in the ID array is not a number or a string.
   */
  public function testValidatedIdsWithInvalidId()
  {
    $subject = new DataValidator();
    $subject->validatedIds([new \DateTime()]);
  }

  /**
   * @expectedException \Vialibri\HarvestFormatting\FormatException
   * @expectedExceptionMessage The value passed as the ID list is not an array.
   */
  public function testValidatedBookDataWithNonArrayForIds()
  {
    $subject = new DataValidator();
    $subject->validatedBookData('Hello', ['ABC123', 1234]);
  }

  /**
   * @expectedException \Vialibri\HarvestFormatting\FormatException
   * @expectedExceptionMessage The value passed as the book list is not an array.
   */
  public function testValidatedBookDataWithNonArrayForBooks()
  {
    $subject = new DataValidator();
    $subject->validatedBookData(['ABC123', 1234], 'Hello');
  }
}
