<?php

namespace Vialibri\HarvestFormatting\Tests\Unit;

use Vialibri\HarvestFormatting\XmlBuilder;

class XmlBuilderTest extends \PHPUnit_Framework_TestCase
{

  public function testBuildIdXmlWithSimpleIds()
  {
    $subject = new XmlBuilder([ [ 'name' => 'Test Source', 'version' => '1.0.0' ] ]);

    $result = $subject->buildIdXml(
      ['ABC123', 'DEFKKK' ],
      '2019-03-25 11:55:59'
    );

    $expected = '<?xml version="1.0" encoding="UTF-8"?>
<Sync_Data>
  <source software="Test Source" version="1.0.0"/>
  <update>
    <date_update>2019-03-25 11:55:59</date_update>
  </update>
  <ID_set>
    <id>ABC123</id>
    <id>DEFKKK</id>
  </ID_set>
</Sync_Data>
';

    $this->assertSame($expected, $result);
  }

  public function testBuildIdXmlWithDateTime()
  {
    $subject = new XmlBuilder([ [ 'name' => 'Test Source', 'version' => '1.0.0' ] ]);

    $result = $subject->buildIdXml(
      ['ABC123', 'DEFKKK' ],
      new \DateTime('2019-03-25 11:55:59')
    );

        $expected = '<?xml version="1.0" encoding="UTF-8"?>
<Sync_Data>
  <source software="Test Source" version="1.0.0"/>
  <update>
    <date_update>2019-03-25 11:55:59</date_update>
  </update>
  <ID_set>
    <id>ABC123</id>
    <id>DEFKKK</id>
  </ID_set>
</Sync_Data>
';

    $this->assertSame($expected, $result);
  }

  public function testBuildBookXml()
  {
    $subject = new XmlBuilder([ [ 'name' => 'Test Source', 'version' => '1.0.0' ] ]);

    $result = $subject->buildBookXml(
      [
        [
          'author' => 'James Dean',
          'title' => 'FooBar',
          'description' => 'Hello & that is what you are looking for.',
          'price' => '34.00',
          'currency' => 'USD',
          'keywords' => [
            'an',
            'array',
            'of',
            'words'
          ],
          'date_update' => '2019-03-25 11:55:59'
        ],
        [
          'author' => 'James Dean',
          'title' => 'Book The Second',
          'description' => 'Kantor',
          'price' => '72.00',
          'description' => 'USD',
          'first_edition' => true,
          'signed' => false,
          'date_update' => '2019-03-20 11:55:59'
        ]
      ],
      'https://www.example.com/data?page=2'
    );

    $expected = '<?xml version="1.0" encoding="UTF-8"?>
<Books next-page="https://www.example.com/data?page=2">
  <source software="Test Source" version="1.0.0"/>
  <Book>
    <author>James Dean</author>
    <title>FooBar</title>
    <description>Hello &amp; that is what you are looking for.</description>
    <price>34.00</price>
    <keywords>an, array, of, words</keywords>
    <date_update>2019-03-25 11:55:59</date_update>
  </Book>
  <Book>
    <author>James Dean</author>
    <title>Book The Second</title>
    <description>USD</description>
    <price>72.00</price>
    <first_edition>yes</first_edition>
    <signed>no</signed>
    <date_update>2019-03-20 11:55:59</date_update>
  </Book>
</Books>
';

    $this->assertSame($expected, $result);
  }

  public function testBuildBookWithDateTimes()
  {
    $subject = new XmlBuilder([ [ 'name' => 'Test Source', 'version' => '1.0.0' ] ]);

    $result = $subject->buildBookXml(
      [
        [
          'author' => 'James Dean',
          'title' => 'FooBar',
          'description' => 'Hello & that is what you are looking for.',
          'price' => '34.00',
          'currency' => 'USD',
          'keywords' => [
            'an',
            'array',
            'of',
            'words'
          ],
          'date_update' => new \DateTime('2019-03-25 11:55:59')
        ],
        [
          'author' => 'James Dean',
          'title' => 'Book The Second',
          'description' => 'Kantor',
          'price' => '72.00',
          'description' => 'USD',
          'first_edition' => true,
          'signed' => false,
          'date_update' => new \DateTime('2019-03-20 11:55:59')
        ]
      ],
      'https://www.example.com/data?page=2'
    );

    $expected = '<?xml version="1.0" encoding="UTF-8"?>
<Books next-page="https://www.example.com/data?page=2">
  <source software="Test Source" version="1.0.0"/>
  <Book>
    <author>James Dean</author>
    <title>FooBar</title>
    <description>Hello &amp; that is what you are looking for.</description>
    <price>34.00</price>
    <keywords>an, array, of, words</keywords>
    <date_update>2019-03-25 11:55:59</date_update>
  </Book>
  <Book>
    <author>James Dean</author>
    <title>Book The Second</title>
    <description>USD</description>
    <price>72.00</price>
    <first_edition>yes</first_edition>
    <signed>no</signed>
    <date_update>2019-03-20 11:55:59</date_update>
  </Book>
</Books>
';

    $this->assertSame($expected, $result);
  }

  public function testBuildErrorXml()
  {
    $subject = new XmlBuilder([ [ 'name' => 'Test Source', 'version' => '1.0.0' ] ]);

    $result = $subject->buildErrorXml('Oh no!');

    $expected = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<error>Oh no!</error>\n";

    $this->assertSame($expected, $result);
  }
}
